#ifndef SimTK_SIMBODY_DECORATION_SUBSYSTEM_H_
#define SimTK_SIMBODY_DECORATION_SUBSYSTEM_H_

/* Portions copyright (c) 2007 Stanford University and Michael Sherman.
 * Contributors:
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, 
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject
 * to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/** @file
 * Define the public interface to DecorationSubsystem, a "do nothing"
 * subsystem providing a place for a MultibodySystem Modeler to toss in some
 * visuals which may optionally be displayed by an application which uses
 * that System.
 */

#include "simbody/internal/common.h"
#include "simbody/internal/System.h"
#include "simbody/internal/Subsystem.h"

#include <cassert>

namespace SimTK {

/**
 * This is the client-side handle class encapsulating the hidden implementation
 * of the DecorationSubsystem. Any Subsystem can generate decorative 
 * geometry, so the methods for extracting the geometry are at the Subsystem
 * level and thus inherited here. So the only significant methods here are
 * those for adding decorative geometry to the System beyond that which is 
 * generated by the other subsystems.
 */
class SimTK_SIMBODY_EXPORT DecorationSubsystem : public Subsystem {
public:
    DecorationSubsystem();

    void addBodyFixedDecoration(BodyId bodyNum, 
                                const Transform& X_GD, 
                                const DecorativeGeometry&);

    void addRubberBandLine(BodyId b1, const Vec3& station1, 
                           BodyId b2, const Vec3& station2,
                           const DecorativeLine&);

    SimTK_PIMPL_DOWNCAST(DecorationSubsystem, Subsystem);
private:
    class DecorationSubsystemRep& updRep();
    const DecorationSubsystemRep& getRep() const;
};

} // namespace SimTK

#endif // SimTK_SIMBODY_DECORATION_SUBSYSTEM_H_
